<?php

namespace Drupal\menu_trail_by_path;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Lock\LockBackendInterface;
use Drupal\Core\Menu\MenuActiveTrail;
use Drupal\Core\Menu\MenuActiveTrailInterface;
use Drupal\Core\Menu\MenuLinkManagerInterface;
use Drupal\Core\Routing\RequestContext;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\menu_link_content\Entity\MenuLinkContent;
use Drupal\menu_trail_by_path\Path\PathHelperInterface;

/**
 * Decorates menu active trail class for the file entity normalizer from HAL.
 */
class MenuTrailByPathActiveTrail extends MenuActiveTrail {

  /**
   * The decorated menu active trail service.
   *
   * @var \Drupal\Core\Menu\MenuActiveTrailInterface
   */
  protected $menuActiveTrail;

  /**
   * The current path helper service.
   *
   * @var \Drupal\menu_trail_by_path\Path\PathHelperInterface
   */
  protected $pathHelper;

  /**
   * The request context service.
   *
   * @var \Drupal\Core\Routing\RequestContext
   */
  protected $context;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Constructs a MenuTrailByPathActiveTrail object.
   *
   * @param \Drupal\Core\Menu\MenuActiveTrailInterface $menu_active_trail
   *   The active menu trail service being decorated.
   * @param \Drupal\Core\Menu\MenuLinkManagerInterface $menu_link_manager
   *   The menu link plugin manager.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   A route match object for finding the active link.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   The cache backend.
   * @param \Drupal\Core\Lock\LockBackendInterface $lock
   *   The lock backend.
   * @param \Drupal\menu_trail_by_path\Path\PathHelperInterface $path_helper
   *   The current path helper service.
   * @param \Drupal\Core\Routing\RequestContext $context
   *   The request context service.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   The language manager.
   */
  public function __construct(MenuActiveTrailInterface $menu_active_trail, MenuLinkManagerInterface $menu_link_manager, RouteMatchInterface $route_match, CacheBackendInterface $cache, LockBackendInterface $lock, PathHelperInterface $path_helper, RequestContext $context, LanguageManagerInterface $languageManager) {
    parent::__construct($menu_link_manager, $route_match, $cache, $lock);
    $this->menuActiveTrail = $menu_active_trail;
    $this->pathHelper = $path_helper;
    $this->context = $context;
    $this->languageManager = $languageManager;
  }

  /**
   * {@inheritdoc}
   *
   * Cannot use decorated service here since we need to look up in our own cache
   * storage, not the decorated service's.
   */
  public function getActiveTrailIds($menu_name) {
    return $this->get($menu_name);
  }

  /**
   * {@inheritdoc}
   */
  public function getActiveLink($menu_name = NULL) {
    return call_user_func_array(
      [$this->menuActiveTrail, 'getActiveLink'],
      func_get_args()
    );
  }

  /**
   * {@inheritdoc}
   *
   * @see https://www.drupal.org/node/2824594
   */
  protected function getCid() {
    if (!isset($this->cid)) {
      return parent::getCid() . ":langcode:{$this->languageManager->getCurrentLanguage()->getId()}:pathinfo:{$this->context->getPathInfo()}";
    }

    return $this->cid;
  }

  /**
   * {@inheritdoc}
   */
  protected function doGetActiveTrailIds($menu_name) {
    // Parent ids; used both as key and value to ensure uniqueness.
    // We always want all the top-level links with parent == ''.
    $active_trail = ['' => ''];

    // If a link in the given menu indeed matches the path, then use it to
    // complete the active trail.
    if ($active_link = $this->getActiveTrailLink($menu_name)) {
      if ($parents = $this->menuLinkManager->getParentIds($active_link->getPluginId())) {
        $active_trail = $parents + $active_trail;
      }
    }

    return $active_trail;
  }

  /**
   * Fetches the deepest, heaviest menu link which matches the deepest trail
   * path url.
   *
   * @param string $menu_name
   *   The menu within which to find the active trail link.
   *
   * @return \Drupal\Core\Menu\MenuLinkInterface|null
   *   The menu link for the given menu, or NULL if there is no matching menu
   *   link.
   */
  public function getActiveTrailLink($menu_name) {
    $trail_urls = $this->pathHelper->getUrls();
    $matches = NULL;

    foreach (array_reverse($trail_urls) as $trail_url) {
      $links = $this->menuLinkManager->loadLinksByRoute($trail_url->getRouteName(), $trail_url->getrouteParameters(), $menu_name);
      if ($links) {
        // Menu link manager sorts ascending by depth, weight, id. Get the
        // last one which should be de deepest menu item.
        $deepest_item = end($links);
        // Comparing the retrieved menu link and the trail url is useful on
        // multilingual sites with multiple menus (using menu_link (field)
        // for example).
        if ($deepest_item->getUrlObject()->toString() == $trail_url->toString()) {
          $matches[] = $deepest_item;
          break;
        }
      }
    }

    if ($matches) {
      $match = $this->getBestMatch($matches, $menu_name);
      return $match;
    }
    return NULL;
  }

  /**
   * Gets for the best match of the menu item.
   *
   * @param \Drupal\Core\Menu\MenuLinkInterface[] $matches
   * @param string $menuName
   *
   * @return \Drupal\Core\Menu\MenuLinkInterface
   */
  private function getBestMatch(array $matches, string $menuName) {
    if (count($matches) === 1) {
      return reset($matches);
    }
    $requestUrl = \Drupal::request()->getRequestUri();

    /** @var \Drupal\Core\Menu\MenuLinkInterface $match */
    foreach ($matches as $match) {
      // Checks if the manually entered menu url is equal with the requested url.
      if (($metadata = $match->getMetaData()) && isset($metadata['entity_id'])) {
        /** @var \Drupal\menu_link_content\Entity\MenuLinkContent $menu_link_content */
        $menu_link_content = MenuLinkContent::load($metadata['entity_id']);
        if ($menu_link_content) {
          $menu_link_parts = array_pad(explode(':', $menu_link_content->get('link')->uri), 2, '');
          if ($requestUrl == $menu_link_parts[1]) {
            return $match;
          }
        }
      }

      // Checks if the requested url is part of the parent url.
      $parentId = $match->getParent();
      if ($parentId) {
        /** @var \Drupal\Core\Menu\MenuLinkInterface $parentMenuLink */
        $parentMenuLink = $this->menuLinkManager->getInstance(['id' => $parentId]);
        if ($parentMenuLink && strpos($requestUrl, $parentMenuLink->getUrlObject()
            ->toString()) === 0) {
          return $match;
        }
      }
    }

    // No matching url found, just return the first one.
    return reset($matches);
  }

}
